<?php
  /**
   * Bugzilla REST API Communication Class
   *
   * @author Quincy Vermue <quincy@ibuildings.nl>
   */
   
  /**
Copyright (c) 2010 Ibuildings.nl BV

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
   */
    
    /**
     * Class BugzillaClient
     *
     * This class consists of all the functions necessary to communicate with
     * BugZilla using the BzAPI.
     *
     * https://wiki.mozilla.org/Bugzilla:REST_API
     *
     * Note: You might have to change the substr in the function: sendTicket()
     *       This is the case if you want the id of your just filed ticket to be
     *       returned only.
     */
    Class BugzillaClient
    {
        private $username;
        private $password;
        private $apiurl;

        /**
         * Constructor
         *
         * @param String $username Your username to login in Bugzilla.
         * @param String $password The password you use to login in Bugzilla.
         * @param String $apiurl The url of your BzAPI.
         */
        public function __construct($username, $password, $apiurl)
        {
            $this->username = $username;
            $this->password = $password;
            $this->apiurl = $apiurl;
        }

        /**
         * Method to connect to the BzAPI using the GET method
         *
         * @param String $method The method of the API you want to use.
         * @return Array
         */
        protected function _connectGet($method)
        {
            $ch = curl_init($this->apiurl.$method.'?username='
                    .$this->username.'&password='.$this->password);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                                                       "Accept: application/json",
                                                       "Content-Type: application/json"
                                                      ));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER  ,1);
            $response = curl_exec($ch);
            curl_close($ch);
            $parsed = json_decode($response);
            return $parsed;
        }

        /**
         * Method to connect to the BzAPI using the POST method
         *
         * @param String $method The method of the API you want to use.
         * @param Array $params The parameters you want to pass to the method.
         * @return Object
         */
        protected function _connectPost($method, $params)
        {
            $data = json_encode($params);
            $ch = curl_init($this->apiurl.$method.'?username='
                    .$this->username.'&password='.$this->password);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                                                        "Accept: application/json",
                                                        "Content-Type: application/json"
                                                       ));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER  ,1);
            $response = curl_exec($ch);
            curl_close($ch);
            return $response;
        }

        /**
         * Method to connect to the BzAPI using the PUT method
         *
         * @param String $method The method of the API you want to use.
         * @param Array $params The parameters you want to pass to the method.
         * @return Object
         */
        protected function _connectPut($method, $params)
        {
            $data = json_encode($params);
            $ch = curl_init($this->apiurl.$method.'?username='
                    .$this->username.'&password='.$this->password);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                                                        "Accept: application/json",
                                                        "Content-Type: application/json"
                                                       ));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER  ,1);
            $response = curl_exec($ch);
            curl_close($ch);
            return $response;
        }

        /**
         * Method to send a ticket to the BzAPI
         *
         * Note: Edit second value of substr($bugid, ##) for your current bzapi location!!
         *       The BzAPI returns ref as: http://someserver:3000/bug/$bugid
         * 
         * @param Array $params The actual bug/ticket.
         * @return String
         */
        public function sendTicket($params)
        {
            $url = $this->_connectPost('bug', $params);
            $result = json_decode($url);
            $bugid = $result->ref;
            return substr($bugid,20);
        }

        /**
         * Method to add a comment to an already existing ticket.
         * Function is a tiny bit slow because it has to do two calls:
         * 1. Get the original bug you want to update in order to get a security token
         * 2. Update the actual ticket including the security token.
         *
         * You need this token, because otherwise Bugzilla wil recognize this as
         * a possible security breach and it will refuse your request.
         *
         * @param String $comment The comment you want to add.
         * @param String $bugid The id of the bug you want to add a comment to.
         * @return Array
         */
        public function addComment($comment, $bugid)
        {
            $method = 'bug/'.$bugid;
            $bug = $this->_connectGet($method);
            $comments[] = array('text' => $comment,
                                );
            $data = array("comments" => $comments, 'token' => $bug->token);
            $response = $this->_connectPut($method, $data);
            return $response;
        }


        /**
         * Method to retrieve a bug from Bugzilla
         *
         * @param Int $bugid The id of the bug you want to retrieve.
         * @return Object
         */
        public function getBug($bugid)
        {
            $method = 'bug/'.$bugid;
            $response = $this->_connectGet($method);
            return $response;
            //use ie $response->reporter->real_name; since this is somehow returned as a json object.
        }

        /**
         * Method to search for a bug
         *
         * You can also use the above _connectGet() instead of file_get_contents.
         * file_get_contents is just shown here to show you another way to perform
         * a GET request on the BzAPI.
         *
         * If you want to use the _connectGet() method, you will have to alter the
         * given url slightly.
         *
         * Your method would look something like the following:
         *
         *      $method = 'bug'.'&'.$param'='.$value;*
         *      $response = $this->_connectGet($method);
         *      $return $response;
         *
         * @param String $param The parameter you want to search for.
         * @param Int $value The value you want to search for.
         * @return Array
         */
        public function search($param, $value)
        {
            $method = 'bug';
            $search_param = '&'.$param.'='.$value;
            $url = $this->apiurl.$method.'?username='.
                    $this->username.'&password='.$this->password.$search_param;
            $response = file_get_contents($url);
            $parsed_response = json_decode($response);
            return $parsed_response;
        }


        /**
         * Retrieve the info of a user, using the BzAPI.
         *
         * When a user provides a username and password, the return Array will
         * contain an extra field called 'can_login' = '1'. This is only the case
         * if the given username/password are allowed to login to Bugzilla.
         *
         * So in a way you can use this method for login and/or user authentication.
         *
         * @param String $username The username of the user you want to retrieve.
         * @return Array 
         */
        public function retrieveUser($username)
        {
            $method = 'user/'.$username;
            $user = $this->connectGet($method);
            return $user;
        }
    }